/* -----------------------------------------------------------------------
 * Title:    8x8 LED dot matrix animations
 * Author:   Alexander Weber alex@tinkerlog.com
 * Date:     21.12.2008
 * Update:	   .11.2010
 * Hardware: ATtiny2313V / ATtiny2313
 * Software: AVRStudio4
 * YOU NEED avr-gcc.exe (GCC) Version 3.4.6 (WinAVR-20060421) for small code
 */

//#include <inttypes.h>
//#include <stdlib.h>
//#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/eeprom.h>
//#include <util/delay.h>
#include <avr/pgmspace.h>
#include "font.h"

// Change these values to adjust scroll speeds and animation iterations
#define ANIMATION_SCROLL_SPEED 100  // how fast to scroll the animations
#define ANIMATION_SPEED 100  		// how fast to change to next picture
#define TEXT_SCROLL_SPEED 120      // how fast to scrill the text (wait)
#define REPEAT_ANIMATION 2        // how often to repeat the animation if in cycling mode
#define REPEAT_TEXT 1              // how often to repeat the text if in cycling mode

// How to add a new message:
// * add the new message (only upper case, see font.h)
// * adjust MAX_MESSAGES
// * add the new message to messages
// NOTE: messages may not be longer than 255 chars.
const prog_char message_00[] = " HTTP://TINKERLOG.COM ";
const prog_char message_01[] = " ZOMBIES AHEAD ";
const prog_char message_02[] = " I AM NO BOMB! ";
const prog_char message_03[] = " 5  4  3  2  1 ... BOOM! ";
const prog_char message_04[] = " !!HAPPY CHRISTMAS!! ";


#define MAX_MESSAGES 5
PGM_P PROGMEM messages[] = {
  message_00
  ,message_01
  ,message_02
  ,message_03
  ,message_04
}; 

//#define MAX_ANIMATIONS 3

const uint8_t sprite_xy[][8] PROGMEM = {
	{ 0x19, 0x2A, 0x7D, 0xFA, 0xFA, 0x7D, 0x2A, 0x19}, // Monster1A
	{ 0x18, 0x2A, 0x7D, 0xF8, 0xF8, 0x7D, 0x2A, 0x18}, // Monster1B
	{ 0x00, 0x00, 0x18, 0x3C, 0x1E, 0x3C, 0x18, 0x00}, // Heart1
	{ 0x38, 0x7C, 0x7E, 0x3F, 0x3F, 0x7E, 0x7C, 0x38}, // Heart2
	{ 0x3F, 0x71, 0xD8, 0x74, 0x74, 0xD8, 0x71, 0x3F}, // Monster2A
	{ 0x3F, 0x19, 0xAC, 0x7E, 0x7E, 0xAC, 0x19, 0x3F}, // Monster2B
	{ 0x06, 0x1E, 0x7E, 0xFF, 0x7E, 0x1E, 0x06, 0x00}, // Baum1
	{ 0x26, 0x9E, 0x7E, 0xFF, 0x7E, 0x9E, 0x06, 0x08}, // Baum1
};

const uint8_t stern1[][8] PROGMEM = {
	{ 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00}, // Star
	{ 0x00, 0x00, 0x18, 0x24, 0x24, 0x18, 0x00, 0x00}, 
	{ 0x00, 0x18, 0x24, 0x42, 0x42, 0x24, 0x18, 0x00}, 
	{ 0x18, 0x24, 0x42, 0x81, 0x81, 0x42, 0x24, 0x18}, 
	{ 0x24, 0x42, 0x81, 0x00, 0x00, 0x81, 0x42, 0x24}, 
	{ 0x42, 0x81, 0x00, 0x00, 0x00, 0x00, 0x81, 0x42}, 
	{ 0x81, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x81}, 
	{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 
};

//next 2 patterns are posted here  http://www.mikroe.com/forum/viewtopic.php?t=11194
const uint8_t stern2[][8] PROGMEM = {
	{ 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00}, // rot. Star
	{ 0x00, 0x00, 0x24, 0x18, 0x18, 0x24, 0x00, 0x00},
	{ 0x00, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x00},
	{ 0x81, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x81},
	{ 0x40, 0x21, 0x12, 0x1C, 0x38, 0x48, 0x84, 0x02},
	{ 0x20, 0x10, 0x09, 0x3A, 0x5C, 0x90, 0x08, 0x04},
	{ 0x10, 0x08, 0x08, 0x79, 0x9E, 0x10, 0x10, 0x08},
	{ 0x08, 0x08, 0x08, 0xF8, 0x1F, 0x10, 0x10, 0x10},
	{ 0x04, 0x08, 0x88, 0x78, 0x1E, 0x11, 0x10, 0x20},
	{ 0x02, 0x84, 0x48, 0x38, 0x1C, 0x12, 0x21, 0x40},
	{ 0x81, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x81},
	{ 0x00, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x00},
	{ 0x00, 0x00, 0x24, 0x18, 0x18, 0x24, 0x00, 0x00},
	{ 0x00, 0x00, 0x00, 0x18, 0x18, 0x00, 0x00, 0x00},
}
;


const uint8_t dancer1[][8] PROGMEM = {
	{ 0x21, 0x63, 0x44, 0x78, 0xF8, 0x44, 0xC3, 0x81},  // dancing man
	{ 0x81, 0xC3, 0x44, 0xF8, 0x78, 0x44, 0x63, 0x21},  
	{ 0x21, 0x63, 0x44, 0x78, 0xF8, 0x44, 0xC3, 0x81},  
	{ 0x81, 0xC3, 0x44, 0xF8, 0x78, 0x44, 0x63, 0x21},  
	{ 0x21, 0x63, 0x44, 0x78, 0xF8, 0x44, 0xC3, 0x81}, 
	{ 0x81, 0xC3, 0x44, 0xF8, 0x78, 0x44, 0x63, 0x21}, 
	{ 0x21, 0x63, 0x44, 0x78, 0xF8, 0x44, 0xC3, 0x81},  
	{ 0x81, 0xC3, 0x44, 0xF8, 0x78, 0x44, 0x63, 0x21},  
	{ 0x81, 0xC3, 0x44, 0xF8, 0x7E, 0x41, 0x61, 0x21},  
	{ 0x21, 0x61, 0x41, 0xFE, 0xFE, 0x41, 0x61, 0x21},  
	{ 0x21, 0x61, 0x41, 0x7E, 0xF8, 0x44, 0xC3, 0x81},  
	{ 0x21, 0x61, 0x41, 0xFE, 0xFE, 0x41, 0x61, 0x21},  
	{ 0x81, 0xC3, 0x44, 0xF8, 0x7E, 0x41, 0x61, 0x21},  
	{ 0x21, 0x61, 0x41, 0xFE, 0xFE, 0x41, 0x61, 0x21},  
	{ 0x81, 0xC3, 0x44, 0xF8, 0xF8, 0x44, 0xC3, 0x81},  
	{ 0x21, 0x61, 0x41, 0xFE, 0xFE, 0x41, 0x61, 0x21}, 
	{ 0x81, 0xC3, 0x44, 0xF8, 0xF8, 0x44, 0xC3, 0x81},  
	{ 0x81, 0xC3, 0x44, 0x78, 0xF8, 0x44, 0xC3, 0x81}, 
	{ 0x81, 0xC3, 0x44, 0xF8, 0x78, 0x44, 0xC3, 0x81}, 
	{ 0x81, 0xC3, 0x44, 0x78, 0xF8, 0x44, 0xC3, 0x81},  
	{ 0x81, 0xC3, 0x44, 0xF8, 0x78, 0x44, 0xC3, 0x81},  
	{ 0x41, 0x43, 0x44, 0xF8, 0x78, 0x44, 0x63, 0x21},  
	{ 0x21, 0x63, 0x44, 0x78, 0xF8, 0x44, 0x43, 0x41},  
	{ 0x41, 0x43, 0x44, 0xF8, 0x78, 0x44, 0x63, 0x21},  
	{ 0x21, 0x63, 0x44, 0x78, 0xF8, 0x44, 0x43, 0x41},  
	{ 0x40, 0x45, 0x4B, 0xF0, 0x78, 0x44, 0x22, 0x21},  
	{ 0x81, 0x47, 0x48, 0xF0, 0x78, 0x44, 0x24, 0x13},  
	{ 0x40, 0x46, 0xFB, 0x78, 0x44, 0x22, 0x21, 0x00}, 
	{ 0x47, 0xF8, 0x7E, 0x41, 0x40, 0x40, 0x00, 0x00},  
	{ 0xF8, 0x7C, 0x43, 0xC1, 0x00, 0x00, 0x00, 0x00},  
	{ 0xFC, 0x42, 0x21, 0x10, 0x00, 0x00, 0x00, 0x00},  
	{ 0x47, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},  
	{ 0x20, 0x20, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00},  
	{ 0x20, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},  
	{ 0x20, 0x20, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00}, 
	{ 0x20, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},  
};


//uint8_t mode_ee EEMEM = 0;                      // stores the mode in eeprom
uint8_t screen_mem[8];			// screen memory
uint8_t active_row=1;			// active row
uint8_t message_ptr = 0;                 // points to the active char in the message
uint8_t message_displayed = 0;           // how often has the message been displayed?
uint8_t active_char = 0;                 // stores the active char
uint8_t message_length = 0;              // stores the length of the active message
uint8_t char_ptr = 0;                    // points to the active col in the char
uint8_t char_length = 0;                 // stores the length of the active char
static volatile uint16_t counter = 0;    // used for delay function


// prototypes
void delay_ms(uint16_t delay);
void copy_to_display(int8_t x, int8_t y, uint8_t sprite[]);
void display_active_row(void);
void show_char();
//void clear_screen(void);
void copy_to_buffer(const prog_uint8_t sprite[8]);
void scroll_animation(const prog_uint8_t sprite_1[], const prog_uint8_t sprite_2[]);

/*
 * ISR TIMER0_OVF_vect
 * Handles overflow interrupts of timer 0.
 *
 * 4MHz
 * ----
 * Prescaler 8 ==> 1953.1 Hz
 * Complete display = 244 Hz
 *
 */


/*
 * display_active_col
 * Deactivates the active column and displays the next one.
 * Data is read from screen_mem.
 *
 *      ATtiny2313
 * 16 - PD0    PB7 - 1
 * 15 - PD1    PB6 - 2
 * 14 - PA1    PB5 - 3
 * 13 - PA0    PB4 - 4
 * 12 - PD2    PB3 - 5
 * 11 - PD3    PB2 - 6
 * 10 - PD4    PB1 - 7
 *  9 - PD5    PB0 - 8
 *
 * NFM-12883 common anode          |
 *     A0B5B4D4B2D3D1D0      +-----+
 * PD5 o o o o o o o o       |     |
 * PA1 o o o o o o o o      _+_    |
 * PB0 o o o o o o o o      \ /    |
 * PD2 o o o o o o o o     __V__   |
 * PB7 o o o o o o o o       |     |
 * PB1 o o o o o o o o    ---+-----C---
 * PB6 o o o o o o o o             |
 * PB3 o o o o o o o o
 *
 * if you have a common cathode matrix rotate your pin layout 90 Degree
 * switch row with column
 */


ISR(TIMER0_OVF_vect) {

  uint8_t row=0,i;

  // timer for delay	
  counter++;

  // shut down all rows and columns
  PORTA = (0 << PA0) | (1 << PA1);
  PORTB = (0 << PB5) | (0 << PB4) | (0 << PB2) | (1 << PB0) | 
          (1 << PB7) | (1 << PB1) | (1 << PB6) | (1 << PB3);
  PORTD = (0 << PD4) | (0 << PD3) | (0 << PD1) | (0 << PD0) |
          (1 << PD5) | (1 << PD2) | (1 << PD6);

  // next row (1,2,4,8,16,32..)
  active_row = (active_row << 7) | (active_row >> 1);

  //row = screen_mem[active_row];
  for (i=0; i<8; i++) {
	row <<= 1;
	if (screen_mem[i] & active_row) { 
		row++ ;
	}
   }

  // output all columns, switch leds on.
  // column 1
  if ((row & 0x80) == 0x80) {
    PORTA |= (1 << PA0);    
  }
  // column 2
  if ((row & 0x40) == 0x40) {
    PORTB |= (1 << PB5);    
  }
  // column 3
  if ((row & 0x20) == 0x20) {
    PORTB |= (1 << PB4);    
  }
  // column 4
  if ((row & 0x10) == 0x10) {
    PORTD |= (1 << PD4);    
  }
  // column 5
  if ((row & 0x08) == 0x08) {
    PORTB |= (1 << PB2);    
  }
  // column 6
  if ((row & 0x04) == 0x04) {
    PORTD |= (1 << PD3);    
  }
  // column 7
  if ((row & 0x02) == 0x02) {
    PORTD |= (1 << PD1);    
  }
  // column 8
  if ((row & 0x01) == 0x01) {
    PORTD |= (1 << PD0);    
  }

  //activate row
  if (active_row==0x80) PORTD &= ~(1 << PD5);
  if (active_row==0x40) PORTA &= ~(1 << PA1);
  if (active_row==0x20) PORTB &= ~(1 << PB0);
  if (active_row==0x10) PORTD &= ~(1 << PD2);
  if (active_row==0x08) PORTB &= ~(1 << PB7);
  if (active_row==0x04) PORTB &= ~(1 << PB1);
  if (active_row==0x02) PORTB &= ~(1 << PB6); 
  if (active_row==0x01) PORTB &= ~(1 << PB3);

}



/*
 * delay_ms
 * Uses the counter that is incremented by the ISR.
 * Max delay is 32767ms.
 */
void delay_ms(uint16_t delay) {
  //while (!(PIND & (1 << PD6))) {}   // used to stop the animation when PD6 goes LOW
  uint16_t t = delay * 2;
  counter = 0;
  while (counter < t) {}
}

/*
 * very nice code but too long
 * copy_to_display
 * Copies sprite data to the screen memory at the given position. 
 */
 /*
void copy_to_display(int8_t x, int8_t y, uint8_t sprite[8]) {
  int8_t i, t;
  uint8_t column;
  clear_screen();
  for (i = 0; i < 8; i++) {
    t = i-y;
    column = ((t >= 0) && (t < 8)) ? sprite[t] : 0x00;
    column = (x >= 0) ? (column >> x) : (column << -x);

	screen_mem[i]=column;
  }
}
*/

/*
 * show_char
 * Displays the actual message. 
 * Scrolls the screen to the left and draws new pixels on the right.
 * char stop bit is bit.7
 */
void show_char(const prog_uint8_t string[]) {
  uint8_t i;
  uint8_t b;

  // shift the screen to the left
  for (i = 0; i < 7; i++) {
    screen_mem[i] = screen_mem[i+1]; 
  }
  // advance a char if needed
  if (char_length == 0x80) {

	char_length =0;	//reset stop bit

    //read next char from progmem
	memcpy_P(&active_char,&string[message_ptr],1);
    message_ptr++;
	
	//string stop byte 0x00 
    if (active_char == 0) {
      message_ptr = 0;
      message_displayed++;
	  char_length =0x80; // immediately read next char
    }

	active_char -= CHAR_OFFSET;
    char_ptr = 0;

	// this makes the space between two chars
	screen_mem[7]=0;
    return; 
  }

  // read pixels for current column of char
  b = pgm_read_byte(&font[active_char * 5 + char_ptr++]);
  //char_length= (b & 0x01);
  //b = (b >> 1);
  char_length= (b & 0x80);
  b = (b & 0x7F);
  // write pixels into screen memory
  screen_mem[7] =b;
}


/*
 * copy_to_buffer
 * Copies the given sprite from PROGMEM to 8x8 LED RAM.
 */
void copy_to_buffer(const prog_uint8_t sprite[8]) {
  memcpy_P(screen_mem, sprite, 8);
}


/*
 * scroll_animation
 * Uses sprite_1 and sprite_2 to draw a simple animation.
 */
void scroll_animation(const prog_uint8_t sprite_1[8], const prog_uint8_t sprite_2[8]) {
  uint8_t i,j;
  int8_t x,z;
  for (i = 0; i < REPEAT_ANIMATION; i++) {
	//you can scroll from right with (x = -8; x < 8; x++)
    for (x = 8; x > -8; x--) {
      //move sprite 1
	  for (j = 0; j < 8; j++) {
			screen_mem[j] = 0x00;
			z = x+j;
			if ((z>=0) && (z<8)) {
			  memcpy_P(&screen_mem[j],&sprite_1[z],1);
			}
	  }
      delay_ms(ANIMATION_SCROLL_SPEED);

      if (x==0) {
		//in the middle shift betwenn sprite_1 and sprite_2
		for (j=0; j<2;j++) {
		    copy_to_buffer(sprite_2);
	    	delay_ms(500);
		    copy_to_buffer(sprite_1);
	    	delay_ms(500);
		}
	  }
	}	
  }
}


int main(void) {

  int8_t i,j,k;

  // timer 0 setup, prescaler 8
  TCCR0B |= (1 << CS01);
 
  // enable timer 0 interrupt
  TIMSK |= (1 << TOIE0);	

  // define outputs
  DDRA |= 0x03;  
  DDRB |= 0xFF;
  DDRD |= 0x3F;
 
/*
  // killed by code size
  // read last mode from eeprom
  // 0 mean cycle through all modes and messages
  mode = eeprom_read_byte(&mode_ee);
  if ((mode == 0) || (mode >= (MAX_ANIMATIONS + MAX_MESSAGES + 1))) {
    mode = 4;
  }
  eeprom_write_byte(&mode_ee, mode + 1);  
*/
  sei();

  while (1) {


//1
     scroll_animation(sprite_xy[0], sprite_xy[1]);

	 scroll_animation(sprite_xy[6], sprite_xy[7]);

//2
	 for (i = 0; i < REPEAT_ANIMATION*2; i++) {
        copy_to_buffer(sprite_xy[3]);
        delay_ms(750);
        copy_to_buffer(sprite_xy[2]);        
        delay_ms(750);
      }

//3
      scroll_animation(sprite_xy[4], sprite_xy[5]);

//4


	for (k = 0; k < REPEAT_ANIMATION; k++) {
      for (i = 0; i < 8; i++) {
        copy_to_buffer(stern1[i]);
        delay_ms(ANIMATION_SPEED);
	  }	
	  for (i = 7; i >= 0; i--) {
        copy_to_buffer(stern1[i]);
        delay_ms(100);
	  }	
	  for (j = 0; j < REPEAT_ANIMATION; j++) {
    	for (i = 0; i < 14; i++) {
        	copy_to_buffer(stern2[i]);
	        delay_ms(ANIMATION_SPEED);
	    }	
	  }
	}


//5
      for (i = 0; i < 36; i++) {
        copy_to_buffer(dancer1[i]);
        delay_ms(350);
	  }	


//6
      for (i = 0; i < MAX_MESSAGES; i++) {
		message_displayed = 0;
		while (message_displayed < REPEAT_TEXT) {
			show_char((uint8_t*)pgm_read_word(&(messages[i])));
			delay_ms(TEXT_SCROLL_SPEED);
		}
	  }

  }

return 0;

}

